//#ifdef HAVE_CONFIG_H
//#include "config.h"
//#endif

#include <dune/solvers/common/wrapownshare.hh>
#include <dune/solvers/iterationsteps/blockgssteps.hh>
#include <dune/solvers/solvers/umfpacksolver.hh>

//#include "solverfactory.hh"

#include "../utils/debugutils.hh"

template <class Functional, class BitVector>
template <class LinearSolver>
SolverFactory<Functional, BitVector>::SolverFactory(
    const Dune::ParameterTree& parset,
    Functional& J,
    LinearSolver&& linearSolver,
    const BitVector& ignoreNodes) :
        J_(Dune::Solvers::wrap_own_share<const Functional>(std::forward<Functional>(J))) {

    //auto localSolver = Dune::TNNMG::gaussSeidelLocalSolver(LocalSolver());
    //nonlinearSmoother_ = std::make_shared<NonlinearSmoother>(*J_, dummyIterate_, localSolver);

    nonlinearSmoother_ = std::make_shared<NonlinearSmoother>(*J_, dummyIterate_, LocalSolver());

    auto linearSolver_ptr = Dune::Solvers::wrap_own_share<std::decay_t<LinearSolver>>(std::forward<LinearSolver>(linearSolver));

    //tnnmgStep_ = std::make_shared<Step>(*J_, dummyIterate_, nonlinearSmoother_, linearSolver_ptr, 10, DefectProjection(), LineSearchSolver());

    tnnmgStep_ = std::make_shared<Step>(*J_, dummyIterate_, nonlinearSmoother_, linearSolver_ptr, DefectProjection(), LineSearchSolver());
    tnnmgStep_->setPreSmoothingSteps(parset.get<int>("main.pre"));
    tnnmgStep_->setIgnore(ignoreNodes);
}

template <class Functional, class BitVector>
void SolverFactory<Functional, BitVector>::setProblem(Vector& x) {
    nonlinearSmoother_->setProblem(x);
    tnnmgStep_->setProblem(x);
}


template <class Functional, class BitVector>
auto SolverFactory<Functional, BitVector>::step()
-> std::shared_ptr<Step> {
    return tnnmgStep_;
}

//#include "solverfactory_tmpl.cc"
